==> etsh-5.2.0.tar.gz: etsh-5.2.0/strtoint.c <==
/*-
 * Copyright (c) 2008-2018
 *	Jeffrey Allen Neitzel <jan (at) etsh (dot) io>.
 *	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JEFFREY ALLEN NEITZEL ``AS IS'', AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL JEFFREY ALLEN NEITZEL BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	@(#)$Id: strtoint.c,v 1.1 2018/06/14 00:00:00 jneitzel $
 */

#include "defs.h"
#include "sh_err.h"
#include "strtoint.h"

bool
strtoint(const char *s, long *l)
{
	long ll;
	char *lbad;
	const char *n;

	n = getmyname();

	errno = 0;
	if (s == NULL || l == NULL) {
		errno = EINVAL;
		fd_print(FD2, FMT3S, n, __func__, strerror(errno));
		if (l != NULL)
			*l = 0;
		return false;
	}

	ll = strtol(s, &lbad, 10);
	if (*s == EOS || (*s == '0' && *(s + 1) >= '0' && *(s + 1) <= '9') ||
	    *lbad != EOS) {
		errno = EINVAL;
		if (EQUAL(n, "if"))
			fd_print(FD2, FMT3S, n, s, ERR_BADINTEGER);
		*l = 0;
		return false;
	}
	if (errno == ERANGE) {
		errno = ERANGE;
		if (EQUAL(n, "if"))
			fd_print(FD2, FMT3S, n, s, ERR_RANGE);
		*l = 0;
		return false;
	}

	*l = ll;
	return true;
}

==> etsh-5.4.0.tar.gz: etsh-5.4.0/strtoint.c <==
/*-
 * Copyright (c) 2008-2019
 *	Jeffrey Allen Neitzel <jan (at) etsh (dot) nl>.
 *	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JEFFREY ALLEN NEITZEL ``AS IS'', AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL JEFFREY ALLEN NEITZEL BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	@(#)$Id: strtoint.c,v 1.5 2019/01/12 23:34:36 jneitzel Exp $
 */

#include "defs.h"
#include "sh_err.h"
#include "strtoint.h"

bool
strtoint(const char *s, long int *l)
{
	long int li;
	char *lbad;
	const char *n;

	n = getmyname();

	errno = 0;
	if (s == NULL || l == NULL) {
		errno = EINVAL;
		fd_print(FD2, FMT3S, n, __func__, strerror(errno));
		if (l != NULL)
			*l = 0;
		return false;
	}

	li = strtol(s, &lbad, 10);
	if (*s == EOS || (*s == '0' && *(s + 1) >= '0' && *(s + 1) <= '9') ||
	    *lbad != EOS) {
		errno = EINVAL;
		if (EQUAL(n, "if"))
			fd_print(FD2, FMT3S, n, s, ERR_BADINTEGER);
		*l = 0;
		return false;
	}
	if (errno == ERANGE) {
		errno = ERANGE;
		if (EQUAL(n, "if"))
			fd_print(FD2, FMT3S, n, s, ERR_RANGE);
		*l = 0;
		return false;
	}

	*l = li;
	return true;
}
